#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Offline Fault Injection Python Class For Camera FI Demo Tool
"""

import os
import sys
import cv2
from cv_bridge import CvBridge
import numpy as np
from PIL import Image
from imgaug import augmenters as iaa


class OfflineImageFault:
    """
    TOF/RGB Camera Image (Realtime) Fault Library For Camera FI Demo Tool
    ----------------------------------------------------------------
    ### Variables:
        - ndir_name: Normal image directory (input file) name
        - fdir_name: Faulty image directory (output file) name
        - img_name: Image name
        - img_format: Image format (.bmp, .png etc.)
        - camera_type: TOF or RGB
        - fault_type: Choosing fault type
        - fault_rate: Fault rate (%)

    ### TOF Image Faults:
        - Salt&Pepper -> salt_pepper()
        - Gaussian -> gaussian()
        - Poisson -> poisson()

    ### RGB Image Faults:
        - Open -> open_fault()
        - Close -> close_fault()
        - Erosion -> erosion()
        - Dilation -> dilation()
        - Gradient -> gradient()
        - Motion-blur -> motion_blur()
        - Partialloss -> partialloss()

    ###### Created by AKE - 04.11.21
    """
    def __init__(self, ndir_name, fdir_name, img_name, img_format,\
         camera_type, fault_type, fault_rate):

        self.ndir_name = ndir_name
        self.fdir_name = fdir_name
        self.img_name = img_name
        self.img_format = img_format
        self.camera_type = camera_type
        self.fault_type = fault_type
        self.fault_rate = fault_rate
        self.bridge = CvBridge()

    def main(self):
        """Main Function"""
        if self.camera_type == "TOF":
            self.tof_image_fault()
        elif self.camera_type == "RGB":
            self.rgb_image_fault()
        else:
            print("Error")

    def tof_image_fault(self):
        """
        TOF Image Faults:
        - Salt&Pepper -> salt_pepper()
        - Gaussian -> gaussian()
        - Poisson -> poisson()
        """

        try:
            image_file = Image.open(self.ndir_name + self.img_name + self.img_format)
            im_arr = np.asarray(image_file)

            if self.fault_type != "nf":
                if self.fault_type == "s":
                    aug_img = self.salt_pepper(self.fault_rate)
                elif self.fault_type == "g":
                    aug_img = self.gaussian(self.fault_rate)
                elif self.fault_type == "p":
                    aug_img = self.poisson(self.fault_rate)
                else:
                    print("This fault cannot be found. Try again...")
                    sys.exit()

                im_arr = aug_img.augment_image(im_arr)
            image_file = Image.fromarray(im_arr).convert('L')
            image_file = np.array(image_file)
            image_name = str(self.img_name + self.img_format)
            # saving faulty tof image
            cv2.imwrite(os.path.join(self.fdir_name, image_name), image_file)

        except Exception as error_msg:
            print(error_msg)

    def rgb_image_fault(self):
        """
        RGB Image Faults:
            - Open -> open_fault()
            - Close -> close_fault()
            - Erosion -> erosion()
            - Dilation -> dilation()
            - Gradient -> gradient()
            - Motion-blur -> motion_blur()
            - Partialloss -> partialloss()
        """
        # Normally, since the fault rate comes as a percentage (it was arranged for TOF),
        # it is provided to bring that value to the range of 0-20.
        fi_rate = int(self.fault_rate * 20)
        kernel = np.ones((fi_rate,fi_rate),np.uint8)
        try:
            image_file = cv2.imread(self.ndir_name + self.img_name + self.img_format)

            if self.fault_type != "nf":
                if self.fault_type == "o":
                    image_file = self.open_fault(image_file, kernel)
                elif self.fault_type == "c":
                    image_file = self.close_fault(image_file, kernel)
                elif self.fault_type == "e":
                    image_file = self.erosion(image_file, kernel)
                elif self.fault_type == "d":
                    image_file = self.dilation(image_file, kernel)
                elif self.fault_type == "gr":
                    image_file = self.gradient(image_file, kernel)
                elif self.fault_type == "m":
                    image_file = self.motion_blur(image_file, fi_rate)
                elif self.fault_type == "par":
                    image_file = self.partialloss(image_file, kernel)
                else:
                    print("This fault cannot be found. Try again...")
                    sys.exit()

            image_name = str(self.img_name + self.img_format)
            # saving faulty tof image
            cv2.imwrite(os.path.join(self.fdir_name, image_name), image_file)

        except Exception as error_msg:
            print(error_msg)

    ### TOF Faults ###
    @classmethod
    def salt_pepper(cls, fi_rate):
        """Salt&Pepper Noise"""
        aug_img = iaa.SaltAndPepper(p = fi_rate)
        return aug_img

    @classmethod
    def gaussian(cls, fi_rate):
        """Gaussian Noise"""
        aug_img = iaa.AdditiveGaussianNoise(loc=0, scale=fi_rate*255)
        return aug_img

    @classmethod
    def laplacian(cls, fi_rate): # Will Be Added.
        """Laplacian Noise (Under-development)"""
        aug_img = iaa.AdditiveLaplaceNoise(loc=0, scale=fi_rate*255)
        return aug_img

    @classmethod
    def poisson(cls, fi_rate):
        """Poisson Noise"""
        fi_rate = float(fi_rate*100)
        aug_img = iaa.AdditivePoissonNoise(lam=fi_rate, per_channel=True)
        return aug_img

    ### RGB Faults ###
    @classmethod
    def open_fault(cls, img_msg, k):
        """Open FI Method"""
        return cv2.morphologyEx(img_msg, cv2.MORPH_OPEN, k)

    @classmethod
    def close_fault(cls, img_msg, k):
        """Close FI Method"""
        return cv2.morphologyEx(img_msg, cv2.MORPH_CLOSE, k)

    @classmethod
    def dilation(cls, img_msg, k):
        """Dilation FI Method"""
        return cv2.dilate(img_msg,k, iterations = 5)

    @classmethod
    def erosion(cls, img_msg, k):
        """Erosion FI Method"""
        return cv2.erode(img_msg,k, iterations = 5)

    @classmethod
    def gradient(cls, img_msg, k):
        """Gradient FI Method"""
        return cv2.morphologyEx(img_msg, cv2.MORPH_GRADIENT, k)

    @classmethod
    def partialloss(cls, img_msg, kernel):
        """Partialloss FI Method (Under-development)"""

    @classmethod
    def motion_blur(cls, img_msg, fi_rate):
        """Motionblur FI Method"""
        return cv2.blur(img_msg,(fi_rate, fi_rate))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Real-time Fault Injector For Camera FI Demo Tool
"""

from os import listdir
from os.path import isfile, join
import rospy
import cv2
import numpy as np
from cv_bridge import CvBridge
from sensor_msgs.msg import Image
from .class_fi_realtime_ui import RealtimeImageFault as fireal

class RealtimeFaultInjector:
    """
    TOF/RGB Camera Image (Realtime) Fault Library For Camera FI Demo Tool
    ----------------------------------------------------------------
    ### Variables:
        - robot_camera = Robot Camera info
        - publish_camera = Faulty Camera screen (optional)
        - camera_type = Camera Type info
        - fault_type: Choosing fault type
        - fault_rate: Fault rate (%)
        - fi_pub_rate: ROS Publisher rate (Hz)
        - cv2_screen: CV2 screen check (True/False)

    ### TOF Image Faults (Under-development):
        - Salt&Pepper -> salt_pepper()
        - Gaussian -> gaussian()
        - Poisson -> poisson()

    ### RGB Image Faults:
        - Open -> open_fault()
        - Close -> close_fault()
        - Erosion -> erosion()
        - Dilation -> dilation()
        - Gradient -> gradient()
        - Motion-blur -> motion_blur()
        - Partialloss -> partialloss()

    ###### Created by AKE - 04.11.21
    """

    def __init__(self,robot_camera, publish_camera, camera_type, fault_type,\
         fault_rate, fi_pub_rate, cv2_screen=False):

        """Main Function"""
        self.bridge = CvBridge()
        self.kernel = self.fault_rate_calc(fault_rate)
        self.camera_type = camera_type
        self.robot_camera = robot_camera
        self.publish_camera = publish_camera
        self.fault_type = fault_type
        self.fault_rate = fault_rate
        self.cv2_screen = cv2_screen
        self.fi_pub_rate = fi_pub_rate

        rospy.init_node("camera_fault_injection_demo_tool_node",anonymous=True)
        # Robot camera information will come from the interface.
        rospy.Subscriber(robot_camera, Image, self.camera_callback)
        #rospy.Subscriber("right_rokos/color_camera/image_raw", Image, self.camera_callback)
        rospy.spin()

    def camera_callback(self, msg):
        """ROS kamera düğümündeki yayını takip eder."""
        if self.camera_type == "TOF":
            img = self.bridge.imgmsg_to_cv2(msg, '32FC1') #For color cam, use "bgr8"
            #im_arr = np.asarray(img)

        elif self.camera_type == "RGB":
            img = self.bridge.imgmsg_to_cv2(msg, 'bgr8')

        fir = fireal(img, self.kernel, self.fault_type, self.fault_rate)

        if self.fault_type == "Gaussian":
            #img = fir.gaussian()
            print("This fault type will be added.")
        elif self.fault_type == "Poisson":
            #img = fir.poisson()
            print("This fault type will be added.")
        elif self.fault_type == "Salt&Pepper":
            #img = fir.saltpepper()
            print("This fault type will be added.")
        elif self.fault_type == "Open":
            img = fir.open_fault()
        elif self.fault_type == "Close":
            img = fir.close_fault()
        elif self.fault_type == "Dilation":
            img = fir.dilation()
        elif self.fault_type == "Erosion":
            img = fir.erosion()
        elif self.fault_type == "Gradient":
            img = fir.gradient()
        elif self.fault_type == "Moiton-blur":
            img = fir.motion_blur()
        elif self.fault_type == "Partialloss":
            #img = fir.partialloss()
            print("Under-development")
        else:
            pass

        if self.cv2_screen:
            cv2.imshow("CV2 Screen",img)
            cv2.waitKey(1)
            print("test")
            ### If the cv2_screen variable is checked, it will show the printed
            # fault on the cv2 screen.
            #os.system("gnome-terminal -x python cv2_show.py")

        else:
            self.camera_fault_publisher(img, self.robot_camera, self.camera_type, self.fi_pub_rate)

    def camera_fault_publisher(self, msg, robot_camera, camera_type, rate):
        """Delivers the fault-injected broadcast to the ROS node."""
        pub_camera = rospy.Publisher(robot_camera, Image, queue_size=10)
        #pub_camera = rospy.Publisher("right_rokos/color_camera/image_raw", Image, queue_size=10)
        if camera_type == "TOF":
            img = self.bridge.cv2_to_imgmsg(msg, '32FC1')
        elif camera_type == "RGB":
            img = self.bridge.cv2_to_imgmsg(msg, 'bgr8')

        pub_camera.publish(img)
        rate = rospy.Rate(rate)
        rate.sleep()

    @classmethod
    def read_image_list(cls, file_path):
        """
        Saves the names of the images in the image folder in a list.
        """
        onlyfiles = [f for f in listdir(file_path) if isfile(join(file_path, f))]
        image_list = [i.split(".",1)[0] for i in onlyfiles]
        return image_list

    @classmethod
    def fault_rate_calc(cls, fi_rate):
        """
        It is the function where the fault rate is calculated and converted as a kernel.
        """
        fi_rate = int(fi_rate)
        kernel = np.ones((fi_rate,fi_rate),np.uint8)

        return kernel



__author__ = "Alim Kerem Erdogmus"
__version__ = "v1.2.3"
__email__ = "kerem.erdogmus@inovasyonmuhendislik.com"
__status__ = "beta"

from .class_fi_realtime_ui import RealtimeImageFault
from .class_fi_offline_ui import OfflineImageFault
from .offline_fault_injector_ui import main
from .realtime_fault_injector_ui import RealtimeFaultInjector
from .ui_interface import Ui_MainWindow
from camfitool import camfitool_main

import sys
from PyQt5 import QtWidgets

def main():
    app = QtWidgets.QApplication(sys.argv)
    window = camfitool_main.MainWindow()
    sys.exit(app.exec_())
    
main()


# -*- coding: utf-8 -*-

"""
Form implementation generated from reading ui file 'interface_v1_2.ui'

Created by: PyQt5 UI code generator 5.14.1

WARNING! All changes made in this file will be lost!
"""

from PyQt5 import QtCore, QtGui, QtWidgets
from .icons_rc import *

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1160, 665)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.header_frame = QtWidgets.QFrame(self.centralwidget)
        self.header_frame.setStyleSheet("")
        self.header_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.header_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.header_frame.setObjectName("header_frame")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.header_frame)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.header_left_frame = QtWidgets.QFrame(self.header_frame)
        self.header_left_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.header_left_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.header_left_frame.setObjectName("header_left_frame")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.header_left_frame)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout.addWidget(self.header_left_frame, 0, QtCore.Qt.AlignTop)
        self.header_center_frame = QtWidgets.QFrame(self.header_frame)
        self.header_center_frame.setStyleSheet("*{border:none;}")
        self.header_center_frame.setFrameShape(QtWidgets.QFrame.Panel)
        self.header_center_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.header_center_frame.setObjectName("header_center_frame")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.header_center_frame)
        self.horizontalLayout_3.setContentsMargins(6, 6, 0, 6)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.header_icon_label = QtWidgets.QLabel(self.header_center_frame)
        self.header_icon_label.setMaximumSize(QtCore.QSize(256, 16777215))
        self.header_icon_label.setText("")
        self.header_icon_label.setPixmap(QtGui.QPixmap(":/icons/icon_signal.svg"))
        self.header_icon_label.setObjectName("header_icon_label")
        self.horizontalLayout_3.addWidget(self.header_icon_label)
        self.header_title_label = QtWidgets.QLabel(self.header_center_frame)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.header_title_label.setFont(font)
        self.header_title_label.setObjectName("header_title_label")
        self.horizontalLayout_3.addWidget(self.header_title_label, 0, QtCore.Qt.AlignRight|QtCore.Qt.AlignTop)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.horizontalLayout.addWidget(self.header_center_frame, 0, QtCore.Qt.AlignTop)
        self.header_right_frame = QtWidgets.QFrame(self.header_frame)
        self.header_right_frame.setMinimumSize(QtCore.QSize(0, 0))
        self.header_right_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.header_right_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.header_right_frame.setObjectName("header_right_frame")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.header_right_frame)
        self.verticalLayout_10.setContentsMargins(0, 0, 6, 0)
        self.verticalLayout_10.setSpacing(9)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.robot_camera_button = QtWidgets.QPushButton(self.header_right_frame)
        self.robot_camera_button.setStyleSheet("background-color: rgb(6, 37, 98);\n"
"color: rgb(255, 255, 255);")
        self.robot_camera_button.setObjectName("robot_camera_button")
        self.verticalLayout_10.addWidget(self.robot_camera_button)
        self.horizontalLayout.addWidget(self.header_right_frame, 0, QtCore.Qt.AlignRight|QtCore.Qt.AlignVCenter)
        self.verticalLayout.addWidget(self.header_frame)
        self.main_body_frame = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.main_body_frame.sizePolicy().hasHeightForWidth())
        self.main_body_frame.setSizePolicy(sizePolicy)
        self.main_body_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.main_body_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.main_body_frame.setObjectName("main_body_frame")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.main_body_frame)
        self.horizontalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_7.setSpacing(0)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.left_body_cont_frame = QtWidgets.QFrame(self.main_body_frame)
        self.left_body_cont_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.left_body_cont_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.left_body_cont_frame.setObjectName("left_body_cont_frame")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.left_body_cont_frame)
        self.verticalLayout_3.setContentsMargins(9, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.left_upper_frame = QtWidgets.QFrame(self.left_body_cont_frame)
        self.left_upper_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.left_upper_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.left_upper_frame.setObjectName("left_upper_frame")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.left_upper_frame)
        self.verticalLayout_4.setContentsMargins(0, 0, 10, 0)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.image_file_frame = QtWidgets.QFrame(self.left_upper_frame)
        self.image_file_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.image_file_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.image_file_frame.setObjectName("image_file_frame")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.image_file_frame)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.image_file_label = QtWidgets.QLabel(self.image_file_frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.image_file_label.setFont(font)
        self.image_file_label.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.image_file_label.setObjectName("image_file_label")
        self.horizontalLayout_9.addWidget(self.image_file_label)
        self.verticalLayout_4.addWidget(self.image_file_frame)
        self.image_file_tree = QtWidgets.QTreeView(self.left_upper_frame)
        self.image_file_tree.setObjectName("image_file_tree")
        self.verticalLayout_4.addWidget(self.image_file_tree)
        self.fi_file_frame = QtWidgets.QFrame(self.left_upper_frame)
        self.fi_file_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.fi_file_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.fi_file_frame.setObjectName("fi_file_frame")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout(self.fi_file_frame)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.fi_file_label = QtWidgets.QLabel(self.fi_file_frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.fi_file_label.setFont(font)
        self.fi_file_label.setObjectName("fi_file_label")
        self.horizontalLayout_11.addWidget(self.fi_file_label)
        self.verticalLayout_4.addWidget(self.fi_file_frame)
        self.fi_file_tree = QtWidgets.QTreeView(self.left_upper_frame)
        self.fi_file_tree.setObjectName("fi_file_tree")
        self.verticalLayout_4.addWidget(self.fi_file_tree)
        self.verticalLayout_3.addWidget(self.left_upper_frame)
        self.left_bottom_frame = QtWidgets.QFrame(self.left_body_cont_frame)
        self.left_bottom_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.left_bottom_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.left_bottom_frame.setObjectName("left_bottom_frame")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.left_bottom_frame)
        self.horizontalLayout_8.setContentsMargins(0, 6, 10, 6)
        self.horizontalLayout_8.setSpacing(0)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.find_image_file_button = QtWidgets.QPushButton(self.left_bottom_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.find_image_file_button.sizePolicy().hasHeightForWidth())
        self.find_image_file_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.find_image_file_button.setFont(font)
        self.find_image_file_button.setStyleSheet("")
        self.find_image_file_button.setObjectName("find_image_file_button")
        self.horizontalLayout_8.addWidget(self.find_image_file_button)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem1)
        self.find_fi_file_button = QtWidgets.QPushButton(self.left_bottom_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.find_fi_file_button.sizePolicy().hasHeightForWidth())
        self.find_fi_file_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.find_fi_file_button.setFont(font)
        self.find_fi_file_button.setObjectName("find_fi_file_button")
        self.horizontalLayout_8.addWidget(self.find_fi_file_button)
        self.verticalLayout_3.addWidget(self.left_bottom_frame)
        self.horizontalLayout_7.addWidget(self.left_body_cont_frame)
        self.center_body_cont_frame = QtWidgets.QFrame(self.main_body_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.center_body_cont_frame.sizePolicy().hasHeightForWidth())
        self.center_body_cont_frame.setSizePolicy(sizePolicy)
        self.center_body_cont_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.center_body_cont_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.center_body_cont_frame.setObjectName("center_body_cont_frame")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.center_body_cont_frame)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.central_menu_label_frame = QtWidgets.QFrame(self.center_body_cont_frame)
        self.central_menu_label_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.central_menu_label_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.central_menu_label_frame.setObjectName("central_menu_label_frame")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.central_menu_label_frame)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.central_menu_label = QtWidgets.QLabel(self.central_menu_label_frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.central_menu_label.setFont(font)
        self.central_menu_label.setObjectName("central_menu_label")
        self.verticalLayout_6.addWidget(self.central_menu_label)
        self.verticalLayout_5.addWidget(self.central_menu_label_frame, 0, QtCore.Qt.AlignTop)
        self.central_menu_frame = QtWidgets.QFrame(self.center_body_cont_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.central_menu_frame.sizePolicy().hasHeightForWidth())
        self.central_menu_frame.setSizePolicy(sizePolicy)
        self.central_menu_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.central_menu_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.central_menu_frame.setObjectName("central_menu_frame")
        self.gridLayout = QtWidgets.QGridLayout(self.central_menu_frame)
        self.gridLayout.setContentsMargins(6, 0, 10, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.fault_rate_frame = QtWidgets.QFrame(self.central_menu_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fault_rate_frame.sizePolicy().hasHeightForWidth())
        self.fault_rate_frame.setSizePolicy(sizePolicy)
        self.fault_rate_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.fault_rate_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.fault_rate_frame.setObjectName("fault_rate_frame")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout(self.fault_rate_frame)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.fault_rate_slider = QtWidgets.QSlider(self.fault_rate_frame)
        self.fault_rate_slider.setOrientation(QtCore.Qt.Horizontal)
        self.fault_rate_slider.setObjectName("fault_rate_slider")
        self.horizontalLayout_10.addWidget(self.fault_rate_slider)
        self.fault_rate_textbrowser = QtWidgets.QTextBrowser(self.fault_rate_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fault_rate_textbrowser.sizePolicy().hasHeightForWidth())
        self.fault_rate_textbrowser.setSizePolicy(sizePolicy)
        self.fault_rate_textbrowser.setMaximumSize(QtCore.QSize(64, 32))
        self.fault_rate_textbrowser.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.fault_rate_textbrowser.setObjectName("fault_rate_textbrowser")
        self.horizontalLayout_10.addWidget(self.fault_rate_textbrowser)
        self.gridLayout.addWidget(self.fault_rate_frame, 6, 3, 1, 1)
        self.fault_rate_label = QtWidgets.QLabel(self.central_menu_frame)
        self.fault_rate_label.setObjectName("fault_rate_label")
        self.gridLayout.addWidget(self.fault_rate_label, 6, 1, 1, 1)
        self.camera_type_label = QtWidgets.QLabel(self.central_menu_frame)
        self.camera_type_label.setObjectName("camera_type_label")
        self.gridLayout.addWidget(self.camera_type_label, 2, 1, 1, 1)
        self.ros_cam_topic_label = QtWidgets.QLabel(self.central_menu_frame)
        self.ros_cam_topic_label.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.ros_cam_topic_label.setFont(font)
        self.ros_cam_topic_label.setText("")
        self.ros_cam_topic_label.setObjectName("ros_cam_topic_label")
        self.gridLayout.addWidget(self.ros_cam_topic_label, 8, 1, 1, 1)
        self.fault_type_combobox = QtWidgets.QComboBox(self.central_menu_frame)
        self.fault_type_combobox.setObjectName("fault_type_combobox")
        self.gridLayout.addWidget(self.fault_type_combobox, 5, 3, 1, 1)
        self.fault_type_label = QtWidgets.QLabel(self.central_menu_frame)
        self.fault_type_label.setObjectName("fault_type_label")
        self.gridLayout.addWidget(self.fault_type_label, 5, 1, 1, 1)
        self.robot_camera_label = QtWidgets.QLabel(self.central_menu_frame)
        self.robot_camera_label.setObjectName("robot_camera_label")
        self.gridLayout.addWidget(self.robot_camera_label, 7, 1, 1, 1)
        self.camera_type_combobox = QtWidgets.QComboBox(self.central_menu_frame)
        self.camera_type_combobox.setObjectName("camera_type_combobox")
        self.gridLayout.addWidget(self.camera_type_combobox, 2, 3, 1, 1)
        self.robot_camera_combobox = QtWidgets.QComboBox(self.central_menu_frame)
        self.robot_camera_combobox.setObjectName("robot_camera_combobox")
        self.gridLayout.addWidget(self.robot_camera_combobox, 7, 3, 1, 1)
        self.ros_cam_topic_text = QtWidgets.QTextEdit(self.central_menu_frame)
        self.ros_cam_topic_text.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ros_cam_topic_text.sizePolicy().hasHeightForWidth())
        self.ros_cam_topic_text.setSizePolicy(sizePolicy)
        self.ros_cam_topic_text.setMaximumSize(QtCore.QSize(16777215, 32))
        self.ros_cam_topic_text.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.ros_cam_topic_text.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.ros_cam_topic_text.setObjectName("ros_cam_topic_text")
        self.gridLayout.addWidget(self.ros_cam_topic_text, 8, 3, 1, 1)
        self.ros_cam_fi_freq_label = QtWidgets.QLabel(self.central_menu_frame)
        self.ros_cam_fi_freq_label.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.ros_cam_fi_freq_label.setFont(font)
        self.ros_cam_fi_freq_label.setText("")
        self.ros_cam_fi_freq_label.setObjectName("ros_cam_fi_freq_label")
        self.gridLayout.addWidget(self.ros_cam_fi_freq_label, 9, 1, 1, 1)
        self.ros_cam_fi_freq_text = QtWidgets.QTextEdit(self.central_menu_frame)
        self.ros_cam_fi_freq_text.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ros_cam_fi_freq_text.sizePolicy().hasHeightForWidth())
        self.ros_cam_fi_freq_text.setSizePolicy(sizePolicy)
        self.ros_cam_fi_freq_text.setMaximumSize(QtCore.QSize(16777215, 32))
        self.ros_cam_fi_freq_text.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.ros_cam_fi_freq_text.setReadOnly(True)
        self.ros_cam_fi_freq_text.setObjectName("ros_cam_fi_freq_text")
        self.gridLayout.addWidget(self.ros_cam_fi_freq_text, 9, 3, 1, 1)
        self.fi_type_label = QtWidgets.QLabel(self.central_menu_frame)
        self.fi_type_label.setObjectName("fi_type_label")
        self.gridLayout.addWidget(self.fi_type_label, 0, 1, 1, 1)
        self.fi_type_combobox = QtWidgets.QComboBox(self.central_menu_frame)
        self.fi_type_combobox.setObjectName("fi_type_combobox")
        self.gridLayout.addWidget(self.fi_type_combobox, 0, 3, 1, 1)
        self.verticalLayout_5.addWidget(self.central_menu_frame)
        self.horizontalLayout_7.addWidget(self.center_body_cont_frame)
        self.right_body_cont_frame = QtWidgets.QFrame(self.main_body_frame)
        self.right_body_cont_frame.setStyleSheet("")
        self.right_body_cont_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.right_body_cont_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.right_body_cont_frame.setObjectName("right_body_cont_frame")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.right_body_cont_frame)
        self.gridLayout_3.setContentsMargins(0, 0, 9, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.right_bottom_frame = QtWidgets.QFrame(self.right_body_cont_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.right_bottom_frame.sizePolicy().hasHeightForWidth())
        self.right_bottom_frame.setSizePolicy(sizePolicy)
        self.right_bottom_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.right_bottom_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.right_bottom_frame.setObjectName("right_bottom_frame")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.right_bottom_frame)
        self.gridLayout_2.setContentsMargins(0, 20, 0, 20)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.show_fi_plan_details_button = QtWidgets.QPushButton(self.right_bottom_frame)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(False)
        font.setWeight(50)
        self.show_fi_plan_details_button.setFont(font)
        self.show_fi_plan_details_button.setStyleSheet("background-color: rgb(6, 37, 98);\n"
"color: rgb(255, 255, 255);")
        self.show_fi_plan_details_button.setObjectName("show_fi_plan_details_button")
        self.gridLayout_2.addWidget(self.show_fi_plan_details_button, 4, 2, 1, 1)
        self.save_fi_plan_button = QtWidgets.QPushButton(self.right_bottom_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.save_fi_plan_button.sizePolicy().hasHeightForWidth())
        self.save_fi_plan_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(False)
        font.setWeight(50)
        self.save_fi_plan_button.setFont(font)
        self.save_fi_plan_button.setStyleSheet("background-color: rgb(6, 37, 98);\n"
"color: rgb(255, 255, 255);")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/cil-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.save_fi_plan_button.setIcon(icon)
        self.save_fi_plan_button.setObjectName("save_fi_plan_button")
        self.gridLayout_2.addWidget(self.save_fi_plan_button, 3, 2, 1, 1)
        self.apply_fault_button = QtWidgets.QPushButton(self.right_bottom_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.apply_fault_button.sizePolicy().hasHeightForWidth())
        self.apply_fault_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.apply_fault_button.setFont(font)
        self.apply_fault_button.setStyleSheet("background-color: rgb(6, 37, 98);\n"
"color: rgb(255, 255, 255);")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/cil-cloud-upload.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.apply_fault_button.setIcon(icon1)
        self.apply_fault_button.setObjectName("apply_fault_button")
        self.gridLayout_2.addWidget(self.apply_fault_button, 4, 0, 1, 1)
        self.randomize_check = QtWidgets.QCheckBox(self.right_bottom_frame)
        self.randomize_check.setObjectName("randomize_check")
        self.gridLayout_2.addWidget(self.randomize_check, 3, 0, 1, 1)
        self.gridLayout_3.addWidget(self.right_bottom_frame, 2, 0, 1, 1)
        self.right_upper_frame = QtWidgets.QFrame(self.right_body_cont_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.right_upper_frame.sizePolicy().hasHeightForWidth())
        self.right_upper_frame.setSizePolicy(sizePolicy)
        self.right_upper_frame.setMinimumSize(QtCore.QSize(0, 0))
        self.right_upper_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.right_upper_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.right_upper_frame.setObjectName("right_upper_frame")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.right_upper_frame)
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.info_frame = QtWidgets.QFrame(self.right_upper_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.info_frame.sizePolicy().hasHeightForWidth())
        self.info_frame.setSizePolicy(sizePolicy)
        self.info_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.info_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.info_frame.setObjectName("info_frame")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.info_frame)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_8.setSpacing(0)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.info_label_frame = QtWidgets.QFrame(self.info_frame)
        self.info_label_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.info_label_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.info_label_frame.setObjectName("info_label_frame")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout(self.info_label_frame)
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.info_label = QtWidgets.QLabel(self.info_label_frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.info_label.setFont(font)
        self.info_label.setStyleSheet("*{border:none;}")
        self.info_label.setObjectName("info_label")
        self.horizontalLayout_13.addWidget(self.info_label)
        self.verticalLayout_8.addWidget(self.info_label_frame)
        self.verticalLayout_7.addWidget(self.info_frame)
        self.info_text = QtWidgets.QTextEdit(self.right_upper_frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.info_text.sizePolicy().hasHeightForWidth())
        self.info_text.setSizePolicy(sizePolicy)
        self.info_text.setMinimumSize(QtCore.QSize(0, 150))
        self.info_text.setMaximumSize(QtCore.QSize(16777215, 250))
        self.info_text.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.info_text.setReadOnly(True)
        self.info_text.setObjectName("info_text")
        self.verticalLayout_7.addWidget(self.info_text)
        self.fi_plan_frame = QtWidgets.QFrame(self.right_upper_frame)
        self.fi_plan_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.fi_plan_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.fi_plan_frame.setObjectName("fi_plan_frame")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout(self.fi_plan_frame)
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.fi_plan_label = QtWidgets.QLabel(self.fi_plan_frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.fi_plan_label.setFont(font)
        self.fi_plan_label.setObjectName("fi_plan_label")
        self.horizontalLayout_12.addWidget(self.fi_plan_label)
        self.verticalLayout_7.addWidget(self.fi_plan_frame)
        self.fi_plan_tree = QtWidgets.QTreeView(self.right_upper_frame)
        self.fi_plan_tree.setObjectName("fi_plan_tree")
        self.verticalLayout_7.addWidget(self.fi_plan_tree)
        self.gridLayout_3.addWidget(self.right_upper_frame, 0, 0, 1, 1)
        self.progressBar_frame = QtWidgets.QFrame(self.right_body_cont_frame)
        self.progressBar_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.progressBar_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.progressBar_frame.setObjectName("progressBar_frame")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.progressBar_frame)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.progressBar_label = QtWidgets.QLabel(self.progressBar_frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.progressBar_label.setFont(font)
        self.progressBar_label.setObjectName("progressBar_label")
        self.verticalLayout_9.addWidget(self.progressBar_label)
        self.progressBar = QtWidgets.QProgressBar(self.progressBar_frame)
        self.progressBar.setStyleSheet("    border: 2px solid #2196F3;\n"
"    border-radius: 5px;\n"
"    background-color: #E0E0E0;")
        self.progressBar.setProperty("value", 24)
        self.progressBar.setAlignment(QtCore.Qt.AlignCenter)
        self.progressBar.setFormat("")
        self.progressBar.setObjectName("progressBar")
        self.verticalLayout_9.addWidget(self.progressBar)
        self.gridLayout_3.addWidget(self.progressBar_frame, 1, 0, 1, 1)
        self.horizontalLayout_7.addWidget(self.right_body_cont_frame)
        self.verticalLayout.addWidget(self.main_body_frame)
        self.footer_frame = QtWidgets.QFrame(self.centralwidget)
        self.footer_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.footer_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.footer_frame.setObjectName("footer_frame")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.footer_frame)
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_6.setSpacing(0)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.footer_left_frame = QtWidgets.QFrame(self.footer_frame)
        self.footer_left_frame.setStyleSheet("*{border:none;}")
        self.footer_left_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.footer_left_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.footer_left_frame.setObjectName("footer_left_frame")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.footer_left_frame)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.version_info_label = QtWidgets.QLabel(self.footer_left_frame)
        self.version_info_label.setObjectName("version_info_label")
        self.horizontalLayout_4.addWidget(self.version_info_label, 0, QtCore.Qt.AlignLeft)
        self.horizontalLayout_6.addWidget(self.footer_left_frame)
        self.footer_info_frame = QtWidgets.QFrame(self.footer_frame)
        self.footer_info_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.footer_info_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.footer_info_frame.setObjectName("footer_info_frame")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout(self.footer_info_frame)
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.footer_info_label = QtWidgets.QLabel(self.footer_info_frame)
        self.footer_info_label.setText("")
        self.footer_info_label.setObjectName("footer_info_label")
        self.horizontalLayout_14.addWidget(self.footer_info_label)
        self.horizontalLayout_6.addWidget(self.footer_info_frame, 0, QtCore.Qt.AlignHCenter)
        self.footer_right_frame = QtWidgets.QFrame(self.footer_frame)
        self.footer_right_frame.setStyleSheet("")
        self.footer_right_frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.footer_right_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.footer_right_frame.setObjectName("footer_right_frame")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.footer_right_frame)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.about_button = QtWidgets.QPushButton(self.footer_right_frame)
        self.about_button.setStyleSheet("background-color: rgb(6, 37, 98);\n"
"color: rgb(255, 255, 255);")
        self.about_button.setObjectName("about_button")
        self.horizontalLayout_5.addWidget(self.about_button, 0, QtCore.Qt.AlignRight)
        self.help_button = QtWidgets.QPushButton(self.footer_right_frame)
        self.help_button.setStyleSheet("background-color: rgb(6, 37, 98);\n"
"color: rgb(255, 255, 255);")
        self.help_button.setObjectName("help_button")
        self.horizontalLayout_5.addWidget(self.help_button)
        self.horizontalLayout_6.addWidget(self.footer_right_frame, 0, QtCore.Qt.AlignRight)
        self.size_grip = QtWidgets.QFrame(self.footer_frame)
        self.size_grip.setMaximumSize(QtCore.QSize(10, 10))
        self.size_grip.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.size_grip.setFrameShadow(QtWidgets.QFrame.Raised)
        self.size_grip.setObjectName("size_grip")
        self.horizontalLayout_6.addWidget(self.size_grip, 0, QtCore.Qt.AlignBottom)
        self.verticalLayout.addWidget(self.footer_frame, 0, QtCore.Qt.AlignBottom)
        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.header_title_label.setText(_translate("MainWindow", "  Camera Fault Injection Tool"))
        self.robot_camera_button.setText(_translate("MainWindow", "Robot Cam"))
        self.image_file_label.setText(_translate("MainWindow", "Normal Image Folders"))
        self.fi_file_label.setText(_translate("MainWindow", "FI Image Folders"))
        self.find_image_file_button.setText(_translate("MainWindow", "Select Image Folder"))
        self.find_fi_file_button.setText(_translate("MainWindow", "Select FI Image Folder"))
        self.central_menu_label.setText(_translate("MainWindow", "Camera Fault Configuration"))
        self.fault_rate_label.setText(_translate("MainWindow", "Fault Rate (%)"))
        self.camera_type_label.setText(_translate("MainWindow", "Camera Type"))
        self.fault_type_label.setText(_translate("MainWindow", "Fault Type"))
        self.robot_camera_label.setText(_translate("MainWindow", "Robot Camera"))
        self.ros_cam_topic_text.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.ros_cam_fi_freq_text.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.fi_type_label.setText(_translate("MainWindow", "Fault Injection Type   "))
        self.show_fi_plan_details_button.setText(_translate("MainWindow", "Show FIP Details"))
        self.save_fi_plan_button.setText(_translate("MainWindow", "  Save FI Plan  "))
        self.apply_fault_button.setText(_translate("MainWindow", "Apply Fault"))
        self.randomize_check.setText(_translate("MainWindow", "Randomize"))
        self.info_label.setText(_translate("MainWindow", "Info"))
        self.fi_plan_label.setText(_translate("MainWindow", "FI Plans"))
        self.progressBar_label.setText(_translate("MainWindow", "FI Applying Progress"))
        self.version_info_label.setText(_translate("MainWindow", "Version 1.2.3"))
        self.about_button.setText(_translate("MainWindow", "About"))
        self.help_button.setText(_translate("MainWindow", "Help"))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
TOF Fault Injector For Camera FI Demo Tool
"""

import os
from os import listdir
from os.path import isfile, join
import random
from .class_fi_offline_ui import OfflineImageFault as ofi

def main(ndir_name, fdir_name, camera_type, fault_type, fault_rate, randomized):
    """
    Offline Camera Fault Injector UI module takes required variables from
    CamFITool and sends them to class_fi_offline_ui accordingly. This method,
    which is used to print an fault in the offline method (the method where
    a fault is printed in a ready image folder and saved in another folder),
    compiles the appropriate fault types determined for TOF and RGB image types.
    """

    img_format= str(img_format_finder(ndir_name))
    img_name_list = read_image_list(ndir_name)
    fault_rate = int(fault_rate)/100
    random_value = 0

    ## TOF Fault Types##
    if fault_type == "Gaussian":
        fault_type = "g"
    elif fault_type == "Poisson":
        fault_type = "p"
    elif fault_type == "Salt&Pepper":
        fault_type = "s"
    ## RGB Fault Types ##
    elif fault_type == "Open":
        fault_type = "o"
    elif fault_type == "Close":
        fault_type = "c"
    elif fault_type == "Dilation":
        fault_type = "d"
    elif fault_type == "Erosion":
        fault_type = "e"
    elif fault_type == "Gradient":
        fault_type = "gr"
    elif fault_type == "Motion-blur":
        fault_type = "m"
    elif fault_type == "Partialloss":
        fault_type = "par"
    else:
        pass

    fi_image_name_list = []

    if randomized is False:
        multi_fault_applier(img_name_list, ndir_name, fdir_name, img_format,\
             camera_type, fault_type, fault_rate)
        done = "Full Injection Completed!"
    else:
        length = len(img_name_list)
        random_value = random.randrange(length)
        fi_image_name_list = random_fault_applier(img_name_list, ndir_name,\
             fdir_name, img_format, camera_type, fault_type, fault_rate, random_value)
        done = "Randomized Injection Sequence Completed!\n"+\
            "----------------------------------\nFault Injected Image Value: "+\
                str(random_value)+"/"+str(len(img_name_list))
        fi_image_name_list.sort()

    return done, len(img_name_list), fi_image_name_list

def read_image_list(file_path):
    """
    TOF saves the names of the images in the Image folder in a list.
    """
    onlyfiles = [f for f in listdir(file_path) if isfile(join(file_path, f))]
    image_list = [i.split(".",1)[0] for i in onlyfiles]
    return image_list

def multi_fault_applier(img_name_list, ndir_name, fdir_name, img_format, camera_type,\
     fault_type, fault_rate):
    """
    Allows multiple faults to be applied to a list of images.
    """
    for i, img in enumerate(img_name_list):
        image_name = [i,img]
        apply_fault = ofi(ndir_name, fdir_name, image_name[1], img_format, camera_type,\
             fault_type, fault_rate)

        apply_fault.main()

def random_fault_applier(img_name_list,ndir_name, fdir_name, img_format, camera_type,\
     fault_type, fault_rate, random_value):
    """
    It is a test function for injecting faults on a random number of images.
    """
    fi_image_name_list = []

    for i, img in enumerate(img_name_list):
        image_name = [i, img]
        apply_fault = ofi(ndir_name, fdir_name, image_name[1], img_format, camera_type,\
             "nf", fault_rate)

        if fault_type in {"s", "g", "p"}:
            apply_fault.tof_image_fault()
        else:
            apply_fault.rgb_image_fault()

    i_val = 0
    y_val = 0

    while i_val != random_value:
        x_val = random.randint(1,4)
        image_name= img_name_list[y_val]

        if x_val%2 == 0:
            apply_fault = ofi(ndir_name, fdir_name, image_name, img_format, camera_type,\
                 fault_type, fault_rate)
            fi_image_name_list.append(image_name)

            if fault_type in {"s", "g", "p"}:
                apply_fault.tof_image_fault()
            else:
                apply_fault.rgb_image_fault()

            i_val+=1
        y_val+=1

        if y_val >= len(img_name_list):
            y_val = 0

    return fi_image_name_list

def img_format_finder(ndir_name):
    """
    This function reads the format of the images taken from the normal
    image database, and the faulty images are output in the same format.
    """
    one_image = os.listdir(ndir_name)[0]
    img_format = "." + one_image.split(".",2)[1]

    return img_format
